// Copyright 2020, 2021, 2022, 2023, 2024 Francesco Biscani (bluescarni@gmail.com), Dario Izzo (dario.izzo@gmail.com)
//
// This file is part of the heyoka library.
//
// This Source Code Form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

#ifndef HEYOKA_CONFIG_HPP
#define HEYOKA_CONFIG_HPP

// clang-format off
#define HEYOKA_VERSION_STRING "6.0.0"
#define HEYOKA_VERSION_MAJOR 6
#define HEYOKA_VERSION_MINOR 0
#define HEYOKA_VERSION_PATCH 0
#define HEYOKA_ABI_VERSION 30
// clang-format on

#define HEYOKA_WITH_MPPP
#define HEYOKA_WITH_SLEEF
/* #undef HEYOKA_BUILD_STATIC_LIBRARY */

#if defined(HEYOKA_WITH_MPPP)

#include <mp++/config.hpp>

#if !defined(MPPP_WITH_BOOST_S11N)

#error mp++ must be configured with support for Boost.serialization.

#endif

#if !defined(MPPP_WITH_FMT)

#error mp++ must be configured with support for the fmt library.

#endif

#if defined(MPPP_WITH_QUADMATH)

#define HEYOKA_HAVE_REAL128

#endif

#if defined(MPPP_WITH_MPFR)

#define HEYOKA_HAVE_REAL

#endif

#endif

// NOTE: handy Boost library for this since 1.73:
// https://www.boost.org/doc/libs/1_73_0/libs/predef/doc/index.html
//
// NOTE: it makes sense here to handle only the GCC/MSVC macros here
// (on the assumption that clang is identical to GCC in this respect).
// No point in using macros provided by compilers we do not test on.
#if defined(_ARCH_PPC) || defined(_M_PPC)

#define HEYOKA_ARCH_PPC

#endif

#if defined(__arm__) || defined(_M_ARM) || defined(_M_ARMT) || defined(__aarch64__) || defined(_M_ARM64)

#define HEYOKA_ARCH_ARM

#endif

#if defined(__x86_64__) || defined(_M_AMD64) || defined(__i386__) || defined(_M_IX86)

#define HEYOKA_ARCH_X86

#endif

// Setup of the ABI versioning and tagging
// machinery.

#if defined(__GNUC__) || defined(__clang__)

#define HEYOKA_ABI_TAG_ATTR __attribute__((abi_tag))

#else

#define HEYOKA_ABI_TAG_ATTR

#endif

// clang-format off
#define HEYOKA_BEGIN_NAMESPACE                                                                                         \
    namespace heyoka                                                                                                   \
    {                                                                                                                  \
    inline namespace v30 HEYOKA_ABI_TAG_ATTR                                                         \
    {

#define HEYOKA_END_NAMESPACE                                                                                           \
    }                                                                                                                  \
    }
// clang-format on

// C++20 constinit.
// NOTE: this seems to be buggy currently on MSVC:
// https://github.com/bluescarni/mppp/issues/291
#if defined(__cpp_constinit) && (!defined(_MSC_VER) || defined(__clang__))

#define HEYOKA_CONSTINIT constinit

#else

#define HEYOKA_CONSTINIT

#endif

#endif
