# Remember several build options.
set(heyoka_WITH_SLEEF ON)
set(heyoka_WITH_MPPP ON)
set(heyoka_WITH_REAL128 OFF)
set(heyoka_WITH_REAL true)
set(heyoka_ABI_VERSION 30)

# Record the LLVM major version against which
# heyoka was built.
set(heyoka_LLVM_VERSION_MAJOR 19)

# Record also the versions of the public dependencies.
set(heyoka_Boost_VERSION 1.87.0)
set(heyoka_fmt_VERSION 10.2.1)
if(${heyoka_WITH_MPPP})
    set(heyoka_mp++_VERSION 1.0.3)
endif()

# Mandatory public dependencies on Boost and fmt.

# NOTE: we look for Boost in CONFIG mode first, as that has become the official supported way
# of locating Boost in recent Boost/CMake versions. If we fail, we try again in
# MODULE mode as last resort.
find_package(Boost 1.69 QUIET COMPONENTS serialization CONFIG)
if(NOT ${Boost_FOUND})
    find_package(Boost 1.69 QUIET MODULE COMPONENTS serialization)
endif()
if(NOT ${Boost_FOUND})
    message(FATAL_ERROR "Could not locate Boost in either CONFIG or MODULE mode.")
endif()

find_package(fmt REQUIRED CONFIG)
if(${fmt_VERSION_MAJOR} VERSION_LESS 9 OR
   ${fmt_VERSION_MAJOR} VERSION_GREATER 11)
   message(FATAL_ERROR "The supported fmt versions are in the range [9, 11], but version ${fmt_VERSION_MAJOR} was found instead.")
endif()

# Optional public dependency on mp++.
if(${heyoka_WITH_MPPP})
    find_package(mp++ 1.0.0 REQUIRED CONFIG)

    # NOTE: this is necessary for the serialisation of several classes.
    if(NOT mp++_WITH_BOOST_S11N)
        message(FATAL_ERROR "mp++ must be installed with support for Boost.serialization.")
    endif()

    # NOTE: needed for formatting numbers.
    if(NOT mp++_WITH_FMT)
        message(FATAL_ERROR "mp++ must be installed with support for fmt.")
    endif()
endif()

# Get current dir.
get_filename_component(_HEYOKA_CONFIG_SELF_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

include(${_HEYOKA_CONFIG_SELF_DIR}/heyoka_export.cmake)

# Clean up.
unset(_HEYOKA_CONFIG_SELF_DIR)
